@@ -1,24 +1,18 @@
 package Template::Plugin::ListOps;
-# Copyright (c) 2007-2007 Sullivan Beck. All rights reserved.
+# Copyright (c) 2007-2008 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
-###############################################################################
-# HISTORY
 ###############################################################################
 
-# Version 1.00  2007-08-16
-#    Initial release
-
-$VERSION = "1.00";
-###############################################################################
+$VERSION = "1.02";
 
 require 5.004;
 
+use warnings;
 use strict;
 use base qw( Template::Plugin );
-use Date::Manip;
-Date_Init();
+use Sort::DataTypes;
 
 =pod
 
@@ -32,30 +26,31 @@ Template::Plugin::ListOps - Plugin interface to list operations
 
 =head1 DESCRIPTION
 
-The ListOps plugin provides some nice list operations for use within
-templates. I wanted to use the Set::Array module to do most of the
-real work, but it fails due to a weakness in the Want module.
-When/if this is fixed, I will probably replace the routines with
-calls to that module.
+The ListOps plugin attempts to provide a complete set of list
+operations for use within templates. Initially, I intended this to be
+a wrapper around the Set::Array module, so much of it's functionality
+and the naming of the functions come from that. However, the
+Set::Array module fails due to a weakness in the Want module, so I
+ended up not using that module at all.
 
 I realize that many of these methods already exist partially as list
-virtual methods, but this is an attempt to have most of the common
-(or not so common) list operations in one place. Also, these also
-have a much more complete handling of duplicate list elements than
-those in the virtual methods.
+virtual methods, but this is an attempt to have all of the common (or
+not so common) list operations in one place. Also, these also have a
+much more complete handling of duplicate list elements than those in
+the virtual methods.
 
 =head1 METHODS
 
 Template::Plugin::ListOps makes the following methods available:
 
-=over 4
-
 =cut
 
 ###############################################################################
 ###############################################################################
 =pod
 
+=over 4
+
 =item unique
 
    [% list = ListOps.unique(list) %]
@@ -222,6 +217,7 @@ sub intersection {
 
    my @ret;
    foreach my $ele (@$list1) {
+      next  if (! defined $ele);
       if (exists $list2{$ele}  &&  $list2{$ele} > 0) {
          $list2{$ele}--;
          push(@ret,$ele);
@@ -336,54 +332,51 @@ sub at {
 =item sorted
 
    [% list = ListOps.sorted(list) %]
-   [% list = ListOps.sorted(list,method) %]
+   [% list = ListOps.sorted(list,method [,arg,arg,...]) %]
 
-This returns the elements of the list sorted based on a method. The
-following methods are known:
+This returns the elements of the list sorted based on a method. Sorting
+is done using the methods defined in the Sort::DataTypes module, refer
+to that manual for a list of methods, and the description of the
+method. For example, to use the sort_domain method, use the call:
 
-   forward   : alphabetical order
-   reverse   : reverse alphabetical order
-   forw_num  : numerical order
-   rev_num   : reverse numerical order
-   random    : sorts them in random order
+   [% list = ListOps.sorted(list,'domain') %]
 
-The following methods are also available for specific data types:
+All methods are available.
 
-   ip        : sort IPs
-   dates     : sort dates in chronological order
-   rev_dates : sort dates in reverse chronological order.
+The following methods are available for backwards compatibility:
 
-If method is not given, it defaults to forward.
+   forward   : same as alphabetic
+   reverse   : same as rev_alphabetic
+   forw_num  : same as numerical
+   rev_num   : same as rev_numerical
+   dates     : same as date
+   rev_dates : same as rev_date
+
+If method is not given, it defaults to alphabetic.
 
 =cut
 
 sub sorted {
-   shift;
-   my $list = shift;
-   my $meth = shift;
-   $meth    = "forward"  if (! $meth);
+   my(@args) = @_;
+   shift @args;
+   my $list = shift @args;
+   my $meth = shift @args;
 
-   my @list;
-   if      ($meth eq "forward") {
-      @list = sort @$list;
-   } elsif ($meth eq "reverse") {
-      @list = sort { $b cmp $a } @$list;
-   } elsif ($meth eq "forw_num") {
-      @list = sort { $a <=> $b } @$list;
-   } elsif ($meth eq "rev_num") {
-      @list = sort { $b <=> $a } @$list;
-   } elsif ($meth eq "random") {
-      @list = @$list;
-      Shuffle(\@list);
-   } elsif ($meth eq "ip") {
-      @list = @$list;
-      SortIP(\@list);
-   } elsif ($meth eq "dates"    ||  $meth eq "rev_dates") {
-      @list = @$list;
-      SortDates(\@list);
-      @list = reverse(@list)  if ($meth eq "rev_dates");
+   $meth    = "alphabetic"  if (! $meth);
+
+   my %meth = qw(forward       alphabetic
+                 reverse       rev_alphabetic
+                 forw_num      numerical
+                 rev_num       rev_numerical
+                 dates         date
+                 rev_dates     rev_date);
+   if (exists $meth{$meth}) {
+      $meth=$meth{$meth};
    }
 
+   sort_by_method($meth,$list,@args);
+
+   my @list = @$list;
    return [ @list ];
 }
 
@@ -509,6 +502,8 @@ sub pushval {
 
    [% ele = ListOps.minval(list) %]
    [% ele = ListOps.maxval(list) %]
+   [% ele = ListOps.minalph(list) %]
+   [% ele = ListOps.maxalph(list) %]
 
 These return the minimum or maximum numerical value in list or
 the first and last values in an alphabetically sorted list.
@@ -672,8 +667,16 @@ sub count {
 =item delete
 
    [% list = ListOps.delete(list,val) %]
+   [% list = ListOps.delete(list,val,op) %]
 
-This deletes all occurences of val from the list.
+This deletes occurences of val from the list.
+
+If op is not given, it defaults to "unique".
+
+If op is "unique", every occurence of val is removed from the list.
+
+If op is "duplicates", duplicates are allowed and only the first
+occurence of val is removed from the list.
 
 =cut
 
@@ -681,10 +684,17 @@ sub delete {
    shift;
    my $list = shift;
    my $val  = shift;
+   my $op   = shift;
+   $op      = "unique"  if (! $op);
 
    my @ret;
+   my $add  = 0;
    foreach my $ele (@$list) {
-      push(@ret,$ele)  unless ($ele eq $val);
+      if ($ele ne $val  ||  $add == 1) {
+         push(@ret,$ele);
+         next;
+      }
+      $add = 1  if ($op eq "duplicates");
    }
    return [ @ret ];
 }
@@ -889,63 +899,6 @@ sub set {
    return [ @list ];
 }
 
-###############################################################################
-###############################################################################
-# ROUTINES FROM MY PERSONAL LIBRARY
-
-{
-   my $randomized = 0;
-
-   sub Randomize {
-      return  if ($randomized);
-      $randomized = 1;
-      srand(time);
-   }
-}
-
-# This routine was taken from The Perl Cookbook
-sub Shuffle {
-  my $array = shift;
-  Randomize();
-  my $i;
-  for ($i = @$array; --$i; ) {
-    my $j = int rand ($i+1);
-    next if $i == $j;
-    @$array[$i,$j] = @$array[$j,$i];
-  }
-}
-
-sub SortIP {
-  my($list)=@_;
-  my(@list)=@$list;
-  @list=sort _SortIP @$list;
-  @$list=@list;
-}
-sub _SortIP {
-  my(@a,@b);
-  (@a)=split('\.',$a);
-  (@b)=split('\.',$b);
-  return ($a[0] <=> $b[0]  ||
-          $a[1] <=> $b[1]  ||
-          $a[2] <=> $b[2]  ||
-          $a[3] <=> $b[3]);
-}
-
-sub SortDates {
-  my($list)=@_;
-  my(@list)=@$list;
-  my %dates;
-  foreach my $date (@list) {
-    $dates{ ParseDate($date) } = $date;
-  }
-  my @sorted = sort { Date_Cmp($a,$b) } keys %dates;
-  @list = ();
-  foreach my $date (@sorted) {
-    push(@list,$dates{$date});
-  }
-  @$list=@list;
-}
-
 ###############################################################################
 ###############################################################################
 =pod
@@ -956,6 +909,11 @@ sub SortDates {
 
 None at this point.
 
+=head1 LICENSE
+
+This script is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
 =head1 AUTHOR
 
 Sullivan Beck (sbeck@cpan.org)
@@ -964,5 +922,12 @@ Sullivan Beck (sbeck@cpan.org)
 
 1;
 # Local Variables:
+# mode: cperl
 # indent-tabs-mode: nil
+# cperl-indent-level: 3
+# cperl-continued-statement-offset: 2
+# cperl-continued-brace-offset: 0
+# cperl-brace-offset: 0
+# cperl-brace-imaginary-offset: 0
+# cperl-label-offset: -2
 # End:
